@@ -59,6 +59,13 @@ from google.cloud.binaryauthorization_v1.services.validation_helper_v1 import (
 )
 from google.cloud.binaryauthorization_v1.types import service
 
+CRED_INFO_JSON = {
+    "credential_source": "/path/to/file",
+    "credential_type": "service account credentials",
+    "principal": "service-account@example.com",
+}
+CRED_INFO_STRING = json.dumps(CRED_INFO_JSON)
+
 
 async def mock_async_gen(data, chunk_size=1):
     for i in range(0, len(data)):  # pragma: NO COVER
@@ -330,6 +337,49 @@ def test__get_universe_domain():
     assert str(excinfo.value) == "Universe Domain cannot be an empty string."
 
 
+@pytest.mark.parametrize(
+    "error_code,cred_info_json,show_cred_info",
+    [
+        (401, CRED_INFO_JSON, True),
+        (403, CRED_INFO_JSON, True),
+        (404, CRED_INFO_JSON, True),
+        (500, CRED_INFO_JSON, False),
+        (401, None, False),
+        (403, None, False),
+        (404, None, False),
+        (500, None, False),
+    ],
+)
+def test__add_cred_info_for_auth_errors(error_code, cred_info_json, show_cred_info):
+    cred = mock.Mock(["get_cred_info"])
+    cred.get_cred_info = mock.Mock(return_value=cred_info_json)
+    client = ValidationHelperV1Client(credentials=cred)
+    client._transport._credentials = cred
+
+    error = core_exceptions.GoogleAPICallError("message", details=["foo"])
+    error.code = error_code
+
+    client._add_cred_info_for_auth_errors(error)
+    if show_cred_info:
+        assert error.details == ["foo", CRED_INFO_STRING]
+    else:
+        assert error.details == ["foo"]
+
+
+@pytest.mark.parametrize("error_code", [401, 403, 404, 500])
+def test__add_cred_info_for_auth_errors_no_get_cred_info(error_code):
+    cred = mock.Mock([])
+    assert not hasattr(cred, "get_cred_info")
+    client = ValidationHelperV1Client(credentials=cred)
+    client._transport._credentials = cred
+
+    error = core_exceptions.GoogleAPICallError("message", details=[])
+    error.code = error_code
+
+    client._add_cred_info_for_auth_errors(error)
+    assert error.details == []
+
+
 @pytest.mark.parametrize(
     "client_class,transport_name",
     [
@@ -1785,11 +1835,15 @@ def test_validate_attestation_occurrence_rest_interceptors(null_interceptor):
         transports.ValidationHelperV1RestInterceptor,
         "post_validate_attestation_occurrence",
     ) as post, mock.patch.object(
+        transports.ValidationHelperV1RestInterceptor,
+        "post_validate_attestation_occurrence_with_metadata",
+    ) as post_with_metadata, mock.patch.object(
         transports.ValidationHelperV1RestInterceptor,
         "pre_validate_attestation_occurrence",
     ) as pre:
         pre.assert_not_called()
         post.assert_not_called()
+        post_with_metadata.assert_not_called()
         pb_message = service.ValidateAttestationOccurrenceRequest.pb(
             service.ValidateAttestationOccurrenceRequest()
         )
@@ -1815,6 +1869,10 @@ def test_validate_attestation_occurrence_rest_interceptors(null_interceptor):
         ]
         pre.return_value = request, metadata
         post.return_value = service.ValidateAttestationOccurrenceResponse()
+        post_with_metadata.return_value = (
+            service.ValidateAttestationOccurrenceResponse(),
+            metadata,
+        )
 
         client.validate_attestation_occurrence(
             request,
@@ -1826,6 +1884,7 @@ def test_validate_attestation_occurrence_rest_interceptors(null_interceptor):
 
         pre.assert_called_once()
         post.assert_called_once()
+        post_with_metadata.assert_called_once()
 
 
 def test_initialize_client_w_rest():
