@@ -60,6 +60,13 @@ from google.cloud.binaryauthorization_v1beta1.services.system_policy_v1_beta1 im
 )
 from google.cloud.binaryauthorization_v1beta1.types import resources, service
 
+CRED_INFO_JSON = {
+    "credential_source": "/path/to/file",
+    "credential_type": "service account credentials",
+    "principal": "service-account@example.com",
+}
+CRED_INFO_STRING = json.dumps(CRED_INFO_JSON)
+
 
 async def mock_async_gen(data, chunk_size=1):
     for i in range(0, len(data)):  # pragma: NO COVER
@@ -333,6 +340,49 @@ def test__get_universe_domain():
     assert str(excinfo.value) == "Universe Domain cannot be an empty string."
 
 
+@pytest.mark.parametrize(
+    "error_code,cred_info_json,show_cred_info",
+    [
+        (401, CRED_INFO_JSON, True),
+        (403, CRED_INFO_JSON, True),
+        (404, CRED_INFO_JSON, True),
+        (500, CRED_INFO_JSON, False),
+        (401, None, False),
+        (403, None, False),
+        (404, None, False),
+        (500, None, False),
+    ],
+)
+def test__add_cred_info_for_auth_errors(error_code, cred_info_json, show_cred_info):
+    cred = mock.Mock(["get_cred_info"])
+    cred.get_cred_info = mock.Mock(return_value=cred_info_json)
+    client = SystemPolicyV1Beta1Client(credentials=cred)
+    client._transport._credentials = cred
+
+    error = core_exceptions.GoogleAPICallError("message", details=["foo"])
+    error.code = error_code
+
+    client._add_cred_info_for_auth_errors(error)
+    if show_cred_info:
+        assert error.details == ["foo", CRED_INFO_STRING]
+    else:
+        assert error.details == ["foo"]
+
+
+@pytest.mark.parametrize("error_code", [401, 403, 404, 500])
+def test__add_cred_info_for_auth_errors_no_get_cred_info(error_code):
+    cred = mock.Mock([])
+    assert not hasattr(cred, "get_cred_info")
+    client = SystemPolicyV1Beta1Client(credentials=cred)
+    client._transport._credentials = cred
+
+    error = core_exceptions.GoogleAPICallError("message", details=[])
+    error.code = error_code
+
+    client._add_cred_info_for_auth_errors(error)
+    assert error.details == []
+
+
 @pytest.mark.parametrize(
     "client_class,transport_name",
     [
@@ -1917,10 +1967,14 @@ def test_get_system_policy_rest_interceptors(null_interceptor):
     ) as transcode, mock.patch.object(
         transports.SystemPolicyV1Beta1RestInterceptor, "post_get_system_policy"
     ) as post, mock.patch.object(
+        transports.SystemPolicyV1Beta1RestInterceptor,
+        "post_get_system_policy_with_metadata",
+    ) as post_with_metadata, mock.patch.object(
         transports.SystemPolicyV1Beta1RestInterceptor, "pre_get_system_policy"
     ) as pre:
         pre.assert_not_called()
         post.assert_not_called()
+        post_with_metadata.assert_not_called()
         pb_message = service.GetSystemPolicyRequest.pb(service.GetSystemPolicyRequest())
         transcode.return_value = {
             "method": "post",
@@ -1942,6 +1996,7 @@ def test_get_system_policy_rest_interceptors(null_interceptor):
         ]
         pre.return_value = request, metadata
         post.return_value = resources.Policy()
+        post_with_metadata.return_value = resources.Policy(), metadata
 
         client.get_system_policy(
             request,
@@ -1953,6 +2008,7 @@ def test_get_system_policy_rest_interceptors(null_interceptor):
 
         pre.assert_called_once()
         post.assert_called_once()
+        post_with_metadata.assert_called_once()
 
 
 def test_initialize_client_w_rest():
